@@ -11,6 +11,7 @@ from sentry_sdk import start_transaction
 from sentry_sdk.profiler import (
     GeventScheduler,
     Profile,
+    Scheduler,
     ThreadScheduler,
     extract_frame,
     extract_stack,
@@ -232,37 +233,6 @@ def test_profiles_sampler(
     assert len(items["profile"]) == profile_count
 
 
-@mock.patch("sentry_sdk.profiler.PROFILE_MINIMUM_SAMPLES", 0)
-def test_profile_context(
-    sentry_init,
-    capture_envelopes,
-    teardown_profiling,
-):
-    sentry_init(
-        traces_sample_rate=1.0,
-        _experiments={"profiles_sample_rate": 1.0},
-    )
-
-    envelopes = capture_envelopes()
-
-    with start_transaction(name="profiling"):
-        pass
-
-    items = defaultdict(list)
-    for envelope in envelopes:
-        for item in envelope.items:
-            items[item.type].append(item)
-
-    assert len(items["transaction"]) == 1
-    assert len(items["profile"]) == 1
-
-    transaction = items["transaction"][0]
-    profile = items["profile"][0]
-    assert transaction.payload.json["contexts"]["profile"] == {
-        "profile_id": profile.payload.json["event_id"],
-    }
-
-
 def test_minimum_unique_samples_required(
     sentry_init,
     capture_envelopes,
@@ -469,19 +439,19 @@ def test_extract_frame(get_frame, function):
     extracted_frame = extract_frame(frame, cwd)
 
     # the abs_path should be equal toe the normalized path of the co_filename
-    assert extracted_frame[0] == os.path.normpath(frame.f_code.co_filename)
+    assert extracted_frame["abs_path"] == os.path.normpath(frame.f_code.co_filename)
 
     # the module should be pull from this test module
-    assert extracted_frame[1] == __name__
+    assert extracted_frame["module"] == __name__
 
     # the filename should be the file starting after the cwd
-    assert extracted_frame[2] == __file__[len(cwd) + 1 :]
+    assert extracted_frame["filename"] == __file__[len(cwd) + 1 :]
 
-    assert extracted_frame[3] == function
+    assert extracted_frame["function"] == function
 
     # the lineno will shift over time as this file is modified so just check
     # that it is an int
-    assert isinstance(extracted_frame[4], int)
+    assert isinstance(extracted_frame["lineno"], int)
 
 
 @pytest.mark.parametrize(
@@ -502,31 +472,32 @@ def test_extract_stack_with_max_depth(depth, max_stack_depth, actual_depth):
 
     # increase the max_depth by the `base_stack_depth` to account
     # for the extra frames pytest will add
-    _, stack, _ = extract_stack(
-        frame, os.getcwd(), max_stack_depth=max_stack_depth + base_stack_depth
+    _, stack, frames = extract_stack(
+        frame, max_stack_depth=max_stack_depth + base_stack_depth
     )
     assert len(stack) == base_stack_depth + actual_depth
+    assert len(frames) == base_stack_depth + actual_depth
 
     for i in range(actual_depth):
-        assert stack[i][3] == "get_frame", i
+        assert get_frame_name(frames[i]) == "get_frame", i
 
     # index 0 contains the inner most frame on the stack, so the lamdba
     # should be at index `actual_depth`
     if sys.version_info >= (3, 11):
         assert (
-            stack[actual_depth][3]
+            get_frame_name(frames[actual_depth])
             == "test_extract_stack_with_max_depth.<locals>.<lambda>"
         ), actual_depth
     else:
-        assert stack[actual_depth][3] == "<lambda>", actual_depth
+        assert get_frame_name(frames[actual_depth]) == "<lambda>", actual_depth
 
 
 def test_extract_stack_with_cache():
     frame = get_frame(depth=1)
 
-    prev_cache = extract_stack(frame, os.getcwd())
+    prev_cache = extract_stack(frame)
     _, stack1, _ = prev_cache
-    _, stack2, _ = extract_stack(frame, os.getcwd(), prev_cache)
+    _, stack2, _ = extract_stack(frame, prev_cache)
 
     assert len(stack1) == len(stack2)
     for i, (frame1, frame2) in enumerate(zip(stack1, stack2)):
@@ -658,12 +629,9 @@ def test_thread_scheduler_single_background_thread(scheduler_class):
 )
 @mock.patch("sentry_sdk.profiler.MAX_PROFILE_DURATION_NS", 1)
 def test_max_profile_duration_reached(scheduler_class):
-    sample = [
-        (
-            "1",
-            (("/path/to/file.py", "file", "file.py", "name", 1),),
-        )
-    ]
+    sample = [("1", extract_stack(get_frame()))]
+
+    cwd = os.getcwd()
 
     with scheduler_class(frequency=1000) as scheduler:
         transaction = Transaction(sampled=True)
@@ -672,18 +640,32 @@ def test_max_profile_duration_reached(scheduler_class):
             assert profile.active
 
             # write a sample at the start time, so still active
-            profile.write(profile.start_ns + 0, process_test_sample(sample))
+            profile.write(cwd, profile.start_ns + 0, sample, {})
             assert profile.active
 
             # write a sample at max time, so still active
-            profile.write(profile.start_ns + 1, process_test_sample(sample))
+            profile.write(cwd, profile.start_ns + 1, sample, {})
             assert profile.active
 
             # write a sample PAST the max time, so now inactive
-            profile.write(profile.start_ns + 2, process_test_sample(sample))
+            profile.write(cwd, profile.start_ns + 2, sample, {})
             assert not profile.active
 
 
+class NoopScheduler(Scheduler):
+    def setup(self):
+        # type: () -> None
+        pass
+
+    def teardown(self):
+        # type: () -> None
+        pass
+
+    def ensure_running(self):
+        # type: () -> None
+        pass
+
+
 current_thread = threading.current_thread()
 thread_metadata = {
     str(current_thread.ident): {
@@ -692,6 +674,12 @@ thread_metadata = {
 }
 
 
+sample_stacks = [
+    extract_stack(get_frame(), max_stack_depth=1),
+    extract_stack(get_frame(), max_stack_depth=2),
+]
+
+
 @pytest.mark.parametrize(
     ("samples", "expected"),
     [
@@ -706,17 +694,7 @@ thread_metadata = {
             id="empty",
         ),
         pytest.param(
-            [
-                (
-                    6,
-                    [
-                        (
-                            "1",
-                            (("/path/to/file.py", "file", "file.py", "name", 1),),
-                        )
-                    ],
-                )
-            ],
+            [(6, [("1", sample_stacks[0])])],
             {
                 "frames": [],
                 "samples": [],
@@ -726,27 +704,9 @@ thread_metadata = {
             id="single sample out of range",
         ),
         pytest.param(
-            [
-                (
-                    0,
-                    [
-                        (
-                            "1",
-                            (("/path/to/file.py", "file", "file.py", "name", 1),),
-                        )
-                    ],
-                )
-            ],
+            [(0, [("1", sample_stacks[0])])],
             {
-                "frames": [
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name",
-                        "filename": "file.py",
-                        "lineno": 1,
-                        "module": "file",
-                    },
-                ],
+                "frames": [extract_frame(sample_stacks[0][2][0], os.getcwd())],
                 "samples": [
                     {
                         "elapsed_since_start_ns": "0",
@@ -761,35 +721,11 @@ thread_metadata = {
         ),
         pytest.param(
             [
-                (
-                    0,
-                    [
-                        (
-                            "1",
-                            (("/path/to/file.py", "file", "file.py", "name", 1),),
-                        )
-                    ],
-                ),
-                (
-                    1,
-                    [
-                        (
-                            "1",
-                            (("/path/to/file.py", "file", "file.py", "name", 1),),
-                        )
-                    ],
-                ),
+                (0, [("1", sample_stacks[0])]),
+                (1, [("1", sample_stacks[0])]),
             ],
             {
-                "frames": [
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name",
-                        "filename": "file.py",
-                        "lineno": 1,
-                        "module": "file",
-                    },
-                ],
+                "frames": [extract_frame(sample_stacks[0][2][0], os.getcwd())],
                 "samples": [
                     {
                         "elapsed_since_start_ns": "0",
@@ -809,44 +745,13 @@ thread_metadata = {
         ),
         pytest.param(
             [
-                (
-                    0,
-                    [
-                        (
-                            "1",
-                            (("/path/to/file.py", "file", "file.py", "name1", 1),),
-                        )
-                    ],
-                ),
-                (
-                    1,
-                    [
-                        (
-                            "1",
-                            (
-                                ("/path/to/file.py", "file", "file.py", "name1", 1),
-                                ("/path/to/file.py", "file", "file.py", "name2", 2),
-                            ),
-                        )
-                    ],
-                ),
+                (0, [("1", sample_stacks[0])]),
+                (1, [("1", sample_stacks[1])]),
             ],
             {
                 "frames": [
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name1",
-                        "filename": "file.py",
-                        "lineno": 1,
-                        "module": "file",
-                    },
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name2",
-                        "filename": "file.py",
-                        "lineno": 2,
-                        "module": "file",
-                    },
+                    extract_frame(sample_stacks[0][2][0], os.getcwd()),
+                    extract_frame(sample_stacks[1][2][0], os.getcwd()),
                 ],
                 "samples": [
                     {
@@ -860,131 +765,27 @@ thread_metadata = {
                         "stack_id": 1,
                     },
                 ],
-                "stacks": [[0], [0, 1]],
+                "stacks": [[0], [1, 0]],
                 "thread_metadata": thread_metadata,
             },
-            id="two identical frames",
+            id="two identical stacks",
         ),
-        pytest.param(
-            [
-                (
-                    0,
-                    [
-                        (
-                            "1",
-                            (
-                                ("/path/to/file.py", "file", "file.py", "name1", 1),
-                                (
-                                    "/path/to/file.py",
-                                    "file",
-                                    "file.py",
-                                    "name2",
-                                    2,
-                                    "file",
-                                ),
-                            ),
-                        )
-                    ],
-                ),
-                (
-                    1,
-                    [
-                        (
-                            "1",
-                            (
-                                (
-                                    "/path/to/file.py",
-                                    "file",
-                                    "file.py",
-                                    "name3",
-                                    3,
-                                    "file",
-                                ),
-                                (
-                                    "/path/to/file.py",
-                                    "file",
-                                    "file.py",
-                                    "name4",
-                                    4,
-                                    "file",
-                                ),
-                            ),
-                        )
-                    ],
-                ),
-            ],
-            {
-                "frames": [
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name1",
-                        "filename": "file.py",
-                        "lineno": 1,
-                        "module": "file",
-                    },
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name2",
-                        "filename": "file.py",
-                        "lineno": 2,
-                        "module": "file",
-                    },
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name3",
-                        "filename": "file.py",
-                        "lineno": 3,
-                        "module": "file",
-                    },
-                    {
-                        "abs_path": "/path/to/file.py",
-                        "function": "name4",
-                        "filename": "file.py",
-                        "lineno": 4,
-                        "module": "file",
-                    },
-                ],
-                "samples": [
-                    {
-                        "elapsed_since_start_ns": "0",
-                        "thread_id": "1",
-                        "stack_id": 0,
-                    },
-                    {
-                        "elapsed_since_start_ns": "1",
-                        "thread_id": "1",
-                        "stack_id": 1,
-                    },
-                ],
-                "stacks": [[0, 1], [2, 3]],
-                "thread_metadata": thread_metadata,
-            },
-            id="two unique stacks",
-        ),
-    ],
-)
-@pytest.mark.parametrize(
-    ("scheduler_class",),
-    [
-        pytest.param(ThreadScheduler, id="thread scheduler"),
-        pytest.param(GeventScheduler, marks=requires_gevent, id="gevent scheduler"),
     ],
 )
 @mock.patch("sentry_sdk.profiler.MAX_PROFILE_DURATION_NS", 5)
 def test_profile_processing(
     DictionaryContaining,  # noqa: N803
-    scheduler_class,
     samples,
     expected,
 ):
-    with scheduler_class(frequency=1000) as scheduler:
+    with NoopScheduler(frequency=1000) as scheduler:
         transaction = Transaction(sampled=True)
         with Profile(transaction, scheduler=scheduler) as profile:
             for ts, sample in samples:
                 # force the sample to be written at a time relative to the
                 # start of the profile
                 now = profile.start_ns + ts
-                profile.write(now, process_test_sample(sample))
+                profile.write(os.getcwd(), now, sample, {})
 
             processed = profile.process()
 
